import datetime
import pytz
import shutil
import tempfile

from md_importer.importer import (
    DEFAULT_LANG,
    logger,
)
from md_importer.importer.publish import find_page
from md_importer.importer.repo import Repo
from md_importer.models import (
    ExternalDocsBranch,
    ExternalDocsBranchImportDirective,
    ImportedArticle,
    VersionExtension,
)


def process_branch(branch):
    tempdir = tempfile.mkdtemp()
    repo = Repo(tempdir, branch.origin, branch.branch_name,
                branch.post_checkout_command, branch.version)
    if repo.get() != 0:
        return None
    for directive in ExternalDocsBranchImportDirective.objects.filter(
            external_docs_branch=branch):
        repo.add_directive(directive.import_from,
                           directive.write_to,
                           directive.advertise,
                           directive.template)
    if not repo.execute_import_directives():
        return None
    if not repo.publish():
        return None
    timestamp = datetime.datetime.now(pytz.utc)

    # Update data in ImportedArticle table
    for page in repo.pages:
        url = page.get_absolute_url()
        if ImportedArticle.objects.filter(branch=branch, url=url).count():
            imported_article = ImportedArticle.objects.filter(
                branch=branch, url=url)[0]
            imported_article.last_import = timestamp
            imported_article.save()
        else:
            imported_article, created = ImportedArticle.objects.get_or_create(
                branch=branch,
                url=url,
                last_import=timestamp)

    if repo.version:
        # Updates pages with information about other versions of the article
        ebs = ExternalDocsBranch.objects.filter(origin=repo.origin)
        available_versions = [a.version for a in ebs]
        urls = [i.url for i in ImportedArticle.objects.all()]

        for page in repo.pages:
            url = page.get_absolute_url()
            alternative_urls = [
                {
                    'version': v,
                    'url': url.replace(repo.version, v),
                }
                for v in available_versions if v != repo.version
            ]
            # Check which of the alternative versions of the article
            # actually exist
            for useless_entry in [x for x in alternative_urls
                                  if x['url'] not in urls]:
                alternative_urls.remove(useless_entry)
            draft = page.get_draft_object()
            if not hasattr(draft, 'versionextension'):
                version_ext = VersionExtension(extended_object=draft)
                version_ext.save()
                draft.versionextension = version_ext
            if draft.versionextension.version != repo.version:
                draft.versionextension.version = repo.version
            draft.versionextension.update_alternatives(alternative_urls)
            draft.versionextension.save()
            draft.publish(DEFAULT_LANG)
    else:
        for page in repo.pages:
            if hasattr(page, 'versionextension'):
                page.versionextension.delete()
                page.publish(DEFAULT_LANG)

    # Remove old entries
    for imported_article in ImportedArticle.objects.filter(
            branch=branch, last_import__lt=timestamp):
        page = find_page(imported_article.url)
        if not page:
            logger.error('Page {} not found for deletion.'.format(url))
        else:
            page.delete()
            imported_article.delete()

    # The import is done, now let's clean up.
    imported_page_ids = [p.id for p in repo.pages
                         if p.changed_by in ['python-api', 'script']]
    ImportedArticle.objects.filter(
        branch=branch).filter(id__in=imported_page_ids).delete()

    shutil.rmtree(tempdir)

    return repo
